"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class assignAndCompare {
    constructor() {
        this.warningNodes = [];
        this.name = `Result not used: `;
        this.description = `The result of an operation not used`;
        this.category = categories_1.default.MISC;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        var _a;
        if (((_a = node) === null || _a === void 0 ? void 0 : _a.nodeType) && staticAnalysisCommon_1.isSubScopeWithTopLevelUnAssignedBinOp(node))
            staticAnalysisCommon_1.getUnAssignedTopLevelBinOps(node).forEach((n) => this.warningNodes.push(n));
    }
    report(compilationResults) {
        return this.warningNodes.map((item, i) => {
            return {
                warning: 'A binary operation yields a value that is not used in the following. This is often caused by confusing assignment (=) and comparison (==).',
                location: item.src
            };
        });
    }
}
exports.default = assignAndCompare;
//# sourceMappingURL=assignAndCompare.js.map