"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class blockBlockhash {
    constructor() {
        this.warningNodes = [];
        this.name = `Blockhash usage: `;
        this.description = `Semantics maybe unclear`;
        this.category = categories_1.default.SECURITY;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (node.nodeType === 'FunctionCall' && staticAnalysisCommon_1.isBlockBlockHashAccess(node))
            this.warningNodes.push(node);
    }
    report(compilationResults) {
        return this.warningNodes.map((item, i) => {
            return {
                warning: `use of "blockhash": "blockhash" is used to access the last 256 block hashes. 
                  A miner computes the block hash by "summing up" the information in the current block mined. 
                  By "summing up" the information in a clever way a miner can try to influence the outcome of a transaction in the current block. 
                  This is especially easy if there are only a small number of equally likely outcomes.`,
                location: item.src
            };
        });
    }
}
exports.default = blockBlockhash;
//# sourceMappingURL=blockBlockhash.js.map