"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class blockTimestamp {
    constructor() {
        this.warningNowNodes = [];
        this.warningblockTimestampNodes = [];
        this.name = `Block timestamp: `;
        this.description = `Semantics maybe unclear`;
        this.category = categories_1.default.SECURITY;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (node.nodeType === "Identifier" && staticAnalysisCommon_1.isNowAccess(node))
            this.warningNowNodes.push(node);
        else if (node.nodeType === "MemberAccess" && staticAnalysisCommon_1.isBlockTimestampAccess(node))
            this.warningblockTimestampNodes.push(node);
    }
    report(compilationResults) {
        return this.warningNowNodes.map((item, i) => {
            return {
                warning: `use of "now": "now" does not mean current time. Now is an alias for block.timestamp. 
                  Block.timestamp can be influenced by miners to a certain degree, be careful.`,
                location: item.src,
                more: 'http://solidity.readthedocs.io/en/develop/frequently-asked-questions.html#are-timestamps-now-block-timestamp-reliable'
            };
        }).concat(this.warningblockTimestampNodes.map((item, i) => {
            return {
                warning: `use of "block.timestamp": "block.timestamp" can be influenced by miners to a certain degree. 
                  That means that a miner can "choose" the block.timestamp, to a certain degree, to change the outcome of a transaction in the mined block.`,
                location: item.src,
                more: 'http://solidity.readthedocs.io/en/develop/frequently-asked-questions.html#are-timestamps-now-block-timestamp-reliable'
            };
        }));
    }
}
exports.default = blockTimestamp;
//# sourceMappingURL=blockTimestamp.js.map