"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class deleteDynamicArrays {
    constructor() {
        this.rel = [];
        this.name = `Delete on dynamic array: `;
        this.description = `Use require/assert appropriately`;
        this.category = categories_1.default.GAS;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (staticAnalysisCommon_1.isDeleteOfDynamicArray(node))
            this.rel.push(node);
    }
    report(compilationResults) {
        return this.rel.map((node) => {
            return {
                warning: 'The “delete” operation when applied to a dynamically sized array in Solidity generates code to delete each of the elements contained. If the array is large, this operation can surpass the block gas limit and raise an OOG exception. Also nested dynamically sized objects can produce the same results.',
                location: node.src,
                more: 'http://solidity.readthedocs.io/en/latest/types.html?highlight=array#delete'
            };
        });
    }
}
exports.default = deleteDynamicArrays;
//# sourceMappingURL=deleteDynamicArrays.js.map