"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
class deleteFromDynamicArray {
    constructor() {
        this.relevantNodes = [];
        this.name = `Delete from dynamic array: `;
        this.description = `'delete' on an array leaves a gap`;
        this.category = categories_1.default.MISC;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (staticAnalysisCommon_1.isDeleteFromDynamicArray(node) && !staticAnalysisCommon_1.isMappingIndexAccess(node.subExpression))
            this.relevantNodes.push(node);
    }
    report(compilationResults) {
        return this.relevantNodes.map((node) => {
            return {
                warning: 'Using delete on an array leaves a gap. The length of the array remains the same. If you want to remove the empty position you need to shift items manually and update the length property.\n',
                location: node.src,
                more: 'https://github.com/miguelmota/solidity-idiosyncrasies'
            };
        });
    }
}
exports.default = deleteFromDynamicArray;
//# sourceMappingURL=deleteFromDynamicArray.js.map