"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
class etherTransferInLoop {
    constructor() {
        this.relevantNodes = [];
        this.name = `Ether transfer in loop: `;
        this.description = `Transferring Ether in a for/while/do-while loop`;
        this.category = categories_1.default.GAS;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        let transferNodes = [];
        if (staticAnalysisCommon_1.isLoop(node)) {
            if (node.body && node.body.nodeType === 'Block')
                transferNodes = node.body.statements.filter(child => (child.nodeType === 'ExpressionStatement' &&
                    child.expression.nodeType === 'FunctionCall' && staticAnalysisCommon_1.isTransfer(child.expression.expression)));
            // When loop body is described without braces
            else if (node.body && node.body.nodeType === 'ExpressionStatement' && node.body.expression.nodeType === 'FunctionCall' && staticAnalysisCommon_1.isTransfer(node.body.expression.expression))
                transferNodes.push(node.body);
            if (transferNodes.length > 0) {
                this.relevantNodes.push(...transferNodes);
            }
        }
    }
    report(compilationResults) {
        return this.relevantNodes.map((node) => {
            return {
                warning: 'Ether payout should not be done in a loop: Due to the block gas limit, transactions can only consume a certain amount of gas. The number of iterations in a loop can grow beyond the block gas limit which can cause the complete contract to be stalled at a certain point. If required then make sure that number of iterations are low and you trust each address involved.',
                location: node.src,
                more: 'https://solidity.readthedocs.io/en/latest/security-considerations.html#gas-limit-and-loops'
            };
        });
    }
}
exports.default = etherTransferInLoop;
//# sourceMappingURL=etherTransferInLoop.js.map