"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class guardConditions {
    constructor() {
        this.guards = [];
        this.name = `Guard conditions: `;
        this.description = `Use 'require' and 'assert' appropriately`;
        this.category = categories_1.default.MISC;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (staticAnalysisCommon_1.isRequireCall(node) || staticAnalysisCommon_1.isAssertCall(node))
            this.guards.push(node);
    }
    report(compilationResults) {
        if (this.guards.length > 0) {
            return [{
                    warning: 'Use assert(x) if you never ever want x to be false, not in any circumstance (apart from a bug in your code). Use require(x) if x can be false, due to e.g. invalid input or a failing external component.',
                    more: 'http://solidity.readthedocs.io/en/develop/control-structures.html#error-handling-assert-require-revert-and-exceptions'
                }];
        }
        return [];
    }
}
exports.default = guardConditions;
//# sourceMappingURL=guardConditions.js.map