"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class inlineAssembly {
    constructor() {
        this.inlineAssNodes = [];
        this.name = `Inline assembly: `;
        this.description = `Inline assembly used`;
        this.category = categories_1.default.SECURITY;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (node.nodeType === 'InlineAssembly')
            this.inlineAssNodes.push(node);
    }
    report(compilationResults) {
        return this.inlineAssNodes.map((node) => {
            return {
                warning: `CAUTION: The Contract uses inline assembly, this is only advised in rare cases. 
                  Additionally static analysis modules do not parse inline Assembly, this can lead to wrong analysis results.`,
                location: node.src,
                more: 'http://solidity.readthedocs.io/en/develop/assembly.html#solidity-assembly'
            };
        });
    }
}
exports.default = inlineAssembly;
//# sourceMappingURL=inlineAssembly.js.map