"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class intDivisionTruncate {
    constructor() {
        this.warningNodes = [];
        this.name = `Data Truncated: `;
        this.description = `Division on int/uint values truncates the result`;
        this.category = categories_1.default.MISC;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (staticAnalysisCommon_1.isIntDivision(node))
            this.warningNodes.push(node);
    }
    report(compilationResults) {
        return this.warningNodes.map((item, i) => {
            return {
                warning: 'Division of integer values yields an integer value again. That means e.g. 10 / 100 = 0 instead of 0.1 since the result is an integer again. This does not hold for division of (only) literal values since those yield rational constants.',
                location: item.src
            };
        });
    }
}
exports.default = intDivisionTruncate;
//# sourceMappingURL=intDivisionTruncate.js.map