"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
const abstractAstView_1 = __importDefault(require("./abstractAstView"));
class selfdestruct {
    constructor() {
        this.name = `Selfdestruct: `;
        this.description = `Beware of caller contracts`;
        this.category = categories_1.default.SECURITY;
        this.algorithm = algorithmCategories_1.default.HEURISTIC;
        this.abstractAst = new abstractAstView_1.default();
        this.visit = this.abstractAst.build_visit((node) => staticAnalysisCommon_1.isStatement(node) || (node.nodeType === 'FunctionCall' && staticAnalysisCommon_1.isSelfdestructCall(node)));
        this.report = this.abstractAst.build_report(this._report.bind(this));
    }
    _report(contracts, multipleContractsWithSameName) {
        const warnings = [];
        contracts.forEach((contract) => {
            contract.functions.forEach((func) => {
                let hasSelf = false;
                func.relevantNodes.forEach((node) => {
                    if (staticAnalysisCommon_1.isSelfdestructCall(node)) {
                        warnings.push({
                            warning: 'Use of selfdestruct: can block calling contracts unexpectedly. Be especially careful if this contract is planned to be used by other contracts (i.e. library contracts, interactions). Selfdestruction of the callee contract can leave callers in an inoperable state.',
                            location: node.src,
                            more: 'https://paritytech.io/blog/security-alert.html'
                        });
                        hasSelf = true;
                    }
                    if (staticAnalysisCommon_1.isStatement(node) && hasSelf) {
                        warnings.push({
                            warning: 'Use of selfdestruct: No code after selfdestruct is executed. Selfdestruct is a terminal.',
                            location: node.src,
                            more: 'http://solidity.readthedocs.io/en/develop/introduction-to-smart-contracts.html#self-destruct'
                        });
                        hasSelf = false;
                    }
                });
            });
        });
        return warnings;
    }
}
exports.default = selfdestruct;
//# sourceMappingURL=selfdestruct.js.map