"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
class stringBytesLength {
    constructor() {
        this.name = `String length: `;
        this.description = `Bytes length != String length`;
        this.category = categories_1.default.MISC;
        this.algorithm = algorithmCategories_1.default.EXACT;
        this.stringToBytesConversions = [];
        this.bytesLengthChecks = [];
    }
    visit(node) {
        if (node.nodeType === "FunctionCall" && staticAnalysisCommon_1.isStringToBytesConversion(node))
            this.stringToBytesConversions.push(node);
        else if (node.nodeType === "MemberAccess" && staticAnalysisCommon_1.isBytesLengthCheck(node))
            this.bytesLengthChecks.push(node);
    }
    report(compilationResults) {
        if (this.stringToBytesConversions.length > 0 && this.bytesLengthChecks.length > 0) {
            return [{
                    warning: 'Bytes and string length are not the same since strings are assumed to be UTF-8 encoded (according to the ABI defintion) therefore one character is not nessesarily encoded in one byte of data.',
                    location: this.bytesLengthChecks[0].src,
                    more: 'https://github.com/ethereum/wiki/wiki/Ethereum-Contract-ABI#argument-encoding'
                }];
        }
        else {
            return [];
        }
    }
}
exports.default = stringBytesLength;
//# sourceMappingURL=stringBytesLength.js.map