"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class thisLocal {
    constructor() {
        this.warningNodes = [];
        this.name = `This on local calls: `;
        this.description = `Invocation of local functions via 'this'`;
        this.category = categories_1.default.GAS;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (node.nodeType === 'MemberAccess' && staticAnalysisCommon_1.isThisLocalCall(node))
            this.warningNodes.push(node);
    }
    report(compilationResults) {
        return this.warningNodes.map(function (item, i) {
            return {
                warning: 'Use of "this" for local functions: Never use this to call functions in the same contract, it only consumes more gas than normal local calls.',
                location: item.src,
                more: 'http://solidity.readthedocs.io/en/develop/control-structures.html#external-function-calls'
            };
        });
    }
}
exports.default = thisLocal;
//# sourceMappingURL=thisLocal.js.map