"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
class txOrigin {
    constructor() {
        this.txOriginNodes = [];
        this.name = `Transaction origin: `;
        this.description = `'tx.origin' is used`;
        this.category = categories_1.default.SECURITY;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (staticAnalysisCommon_1.isTxOriginAccess(node))
            this.txOriginNodes.push(node);
    }
    report(compilationResults) {
        return this.txOriginNodes.map((item, i) => {
            return {
                warning: `Use of tx.origin: "tx.origin" is useful only in very exceptional cases. 
                  If you use it for authentication, you usually want to replace it by "msg.sender", because otherwise any contract you call can act on your behalf.`,
                location: item.src
            };
        });
    }
}
exports.default = txOrigin;
//# sourceMappingURL=txOrigin.js.map